@@ -12,36 +12,40 @@
 
 (in-package "SB-VM")
 
-#!-sb-fluid (declaim (inline store-word))
-(defun store-word (word base &optional (offset 0) (lowtag 0))
-  (declare (type (unsigned-byte #.n-word-bits) word base offset)
-           (type (unsigned-byte #.n-lowtag-bits) lowtag))
-  (setf (sap-ref-word (int-sap base) (- (ash offset word-shift) lowtag)) word))
+(!define-load-time-global *allocator-mutex* (sb-thread:make-mutex :name "Allocator"))
 
 (defun allocate-static-vector (widetag length words)
   (declare (type (unsigned-byte #.n-widetag-bits) widetag)
-           (type (unsigned-byte #.n-word-bits) words)
+           (type word words)
            (type index length))
-  ;; WITHOUT-GCING implies WITHOUT-INTERRUPTS
-  (or
-   (without-gcing
-     (let* ((pointer (sap-int *static-space-free-pointer*))
-            (vector (logior pointer other-pointer-lowtag))
-            (nbytes (pad-data-block (+ words vector-data-offset)))
-            (new-pointer (+ pointer nbytes)))
-       (when (> static-space-end new-pointer)
-         (store-word widetag
-                     vector 0 other-pointer-lowtag)
-         (store-word (fixnumize length)
-                     vector vector-length-slot other-pointer-lowtag)
-         (store-word 0 new-pointer)
-         (setf *static-space-free-pointer* (int-sap new-pointer))
-         (%make-lisp-obj vector))))
-   (error 'simple-storage-condition
-          :format-control "Not enough memory left in static space to ~
-                           allocate vector.")))
+  ;; This does not need WITHOUT-GCING, but it will be implicitly wrapped
+  ;; in WITHOUT-INTERRUPTS due to the default behavior of system mutexes,
+  ;; which is important so that we don't leave an inconsistent state.
+  ;; To think about why it is OK to leave GC enabled, consider that
+  ;; neither GC nor another thread will examine static space above the
+  ;; current value of *STATIC-SPACE-FREE-POINTER*.
+  (or (sb-thread::with-system-mutex (*allocator-mutex*)
+        (let* ((pointer *static-space-free-pointer*)
+               (nbytes (pad-data-block (+ words vector-data-offset)))
+               (new-pointer (sap+ pointer nbytes)))
+          (when (sap<= new-pointer (int-sap static-space-end))
+            ;; By storing the length prior to the widetag, the word at the old
+            ;; free pointer decodes as a cons instead of a 0-length vector.
+            ;; Not that it should matter, but it seems slightly better to change
+            ;; the new object atomically to a correctly-sized vector rather than
+            ;; a cons changing into the wrong vector into the right vector.
+            (setf (sap-ref-word pointer (ash vector-length-slot word-shift))
+                  (fixnumize length))
+            ;; then store the widetag
+            (setf (sap-ref-word pointer 0) widetag)
+            ;; then the new free pointer
+            (setf *static-space-free-pointer* new-pointer)
+            (%make-lisp-obj (logior (sap-int pointer) other-pointer-lowtag)))))
+      (error 'simple-storage-condition
+             :format-control
+             "Not enough room left in static space to allocate vector.")))
 
-#!+immobile-space
+#+immobile-space
 (progn
 
 (define-alien-variable varyobj-space-size (unsigned 32))
@@ -52,17 +56,13 @@
 (define-alien-variable ("fixedobj_free_pointer" *fixedobj-space-free-pointer*)
   system-area-pointer)
 
-(!define-load-time-global *immobile-space-mutex*
-    #.(sb-thread:make-mutex :name "Immobile space"))
-
 (eval-when (:compile-toplevel)
   (assert (eql code-boxed-size-slot 1))
   (assert (eql code-debug-info-slot 2)))
 
 (define-alien-variable "varyobj_holes" long)
 (define-alien-variable "varyobj_page_touched_bits" (* (unsigned 32)))
-(define-alien-variable "varyobj_page_scan_start_offset" (* (unsigned 16)))
-(define-alien-variable "varyobj_page_gens" (* (unsigned 8)))
+(define-alien-variable "varyobj_pages" (* (unsigned 32)))
 (define-alien-routine "find_preceding_object" long (where long))
 
 ;;; Lazily created freelist, used only when unallocate is called:
@@ -82,10 +82,22 @@
 (defun varyobj-page-address (index)
   (+ varyobj-space-start (* index immobile-card-bytes)))
 
+(declaim (inline (setf varyobj-page-scan-start-offset)))
+(defun (setf varyobj-page-scan-start-offset) (newval index)
+  ;; NEWVAL is passed in as a byte count but we want to store it as doublewords
+  ;; so it needs right-shifting by 1+ word-shift. However because it is a field
+  ;; of a packed word, it needs left-shifting by 8. We can shift by the net amount
+  ;; provided that no zero bits would be right-shifted out.
+  (aver (zerop (logand newval lowtag-mask)))
+  (setf (deref varyobj-pages index)
+        (logior (ash newval (- 8 (1+ sb-vm:word-shift)))
+                (logand (deref varyobj-pages index) #xFF)))
+  newval)
+
 ;;; Convert a zero-based varyobj page index into a scan start address.
 (defun varyobj-page-scan-start (index)
   (- (+ varyobj-space-start (* (1+ index) immobile-card-bytes))
-     (* 2 n-word-bytes (deref varyobj-page-scan-start-offset index))))
+     (* 2 n-word-bytes (ash (deref varyobj-pages index) -8))))
 
 (declaim (inline hole-p))
 (defun hole-p (raw-address)
@@ -138,7 +150,7 @@
          (table (cdr freelist))
          (old (gethash (hole-size hole) table)))
     ;; Check for double-free error
-    #!+immobile-space-debug (aver (not (member hole (gethash size table))))
+    #+immobile-space-debug (aver (not (member hole (gethash size table))))
     (unless old
       (setf (car freelist)
             (sorted-list-insert size (car freelist) #'identity)))
@@ -152,7 +164,7 @@
          (old-length (length list))
          (new (delete hole list :count 1)))
     (declare (ignorable old-length))
-    #!+immobile-space-debug (aver (= (length new) (1- old-length)))
+    #+immobile-space-debug (aver (= (length new) (1- old-length)))
     (cond (new
            (setf (gethash key table) new))
           (t
@@ -190,7 +202,7 @@
                  n-fixnum-tag-bits)))))
 
 (defun unallocate (hole)
-  #!+immobile-space-debug
+  #+immobile-space-debug
   (awhen *in-use-bits* (mark-range it hole (hole-size hole) nil))
   (let* ((hole-end (hole-end-address hole))
          (end-is-free-ptr (eql hole-end (sap-int *varyobj-space-free-pointer*))))
@@ -210,9 +222,9 @@
           (last-page (varyobj-page-index (1- hole-end))))
       (when (and (eql (varyobj-page-scan-start first-page) hole)
                  (< first-page last-page))
-        (setf (deref varyobj-page-scan-start-offset first-page) 0))
+        (setf (varyobj-page-scan-start-offset first-page) 0))
       (loop for page from (1+ first-page) below last-page
-            do (setf (deref varyobj-page-scan-start-offset page) 0))
+            do (setf (varyobj-page-scan-start-offset page) 0))
       ;; Only touch the offset for the last page if it pointed to this hole.
       ;; If the following object is a hole that is in the pending free list,
       ;; it's ok, but if it's a hole that is already in the freelist,
@@ -225,10 +237,10 @@
                                      ((freed-hole-p hole-end)
                                       (hole-end-address hole-end))
                                      (t hole-end))))
-          (setf (deref varyobj-page-scan-start-offset last-page)
+          (setf (varyobj-page-scan-start-offset last-page)
                 (if (< new-scan-start page-end)
                     ;; Compute new offset backwards relative to the page end.
-                    (/ (- page-end new-scan-start) (* 2 n-word-bytes))
+                    (- page-end new-scan-start)
                     0))))) ; Page becomes empty
 
     (unless *immobile-freelist*
@@ -266,7 +278,7 @@
   (setq n-bytes (align-up n-bytes (* 2 n-word-bytes)))
   ;; Can't allocate fewer than 4 words due to min hole size.
   (aver (>= n-bytes (* 4 n-word-bytes)))
-  (sb-thread::with-system-mutex (*immobile-space-mutex* :without-gcing t)
+  (sb-thread::with-system-mutex (*allocator-mutex* :without-gcing t)
    (unless (zerop varyobj-holes)
      ;; If deferred sweep needs to happen, do so now.
      ;; Concurrency could potentially be improved here: at most one thread
@@ -317,16 +329,14 @@
             (index (varyobj-page-index addr))
             (obj-end (+ addr n-bytes)))
        ;; Mark the page as being used by a nursery object.
-       (setf (deref varyobj-page-gens index)
-             (logior (deref varyobj-page-gens index) 1))
+       (setf (deref varyobj-pages index) (logior (deref varyobj-pages index) 1))
        ;; On the object's first page, set the scan start only if addr
        ;; is lower than the current page-scan-start object.
        ;; Note that offsets are expressed in doublewords backwards from
        ;; page end, so that we can direct the scan start to any doubleword
-       ;; on the page or in the preceding 1MiB (approximately).
+       ;; on the page or in the preceding 256MiB (approximately).
        (when (< addr (varyobj-page-scan-start index))
-         (setf (deref varyobj-page-scan-start-offset index)
-               (ash (- page-end addr) (- (1+ word-shift)))))
+         (setf (varyobj-page-scan-start-offset index) (- page-end addr)))
        ;; On subsequent pages, always set the scan start, since there can not
        ;; be a lower-addressed object touching those pages.
        (loop
@@ -334,15 +344,14 @@
         (incf page-end immobile-card-bytes)
         (incf index)
         (when (>= page-start obj-end) (return))
-        (setf (deref varyobj-page-scan-start-offset index)
-              (ash (- page-end addr) (- (1+ word-shift))))))
-     #!+immobile-space-debug ; "address sanitizer"
+        (setf (varyobj-page-scan-start-offset index) (- page-end addr))))
+     #+immobile-space-debug ; "address sanitizer"
      (awhen *in-use-bits* (mark-range it addr n-bytes t))
      (setf (sap-ref-word (int-sap addr) 0) word0
            (sap-ref-word (int-sap addr) n-word-bytes) word1)
      ;; 0-fill the remainder of the object
-     (#!+64-bit system-area-ub64-fill
-      #!-64-bit system-area-ub32-fill
+     (#+64-bit system-area-ub64-fill
+      #-64-bit system-area-ub32-fill
       0 (int-sap addr) 2 (- (ash n-bytes (- word-shift)) 2))
      ;; Only after making the new object can we reduce the size of the hole
      ;; that contained the new allocation (if it entailed chopping a hole
@@ -376,8 +385,8 @@
   (allocate-immobile-vector simple-array-unsigned-byte-8-widetag n-elements
                             (ceiling n-elements n-word-bytes)))
 (defun allocate-immobile-word-vector (n-elements)
-  (allocate-immobile-vector #!+64-bit simple-array-unsigned-byte-64-widetag
-                            #!-64-bit simple-array-unsigned-byte-32-widetag
+  (allocate-immobile-vector #+64-bit simple-array-unsigned-byte-64-widetag
+                            #-64-bit simple-array-unsigned-byte-32-widetag
                             n-elements n-elements))
 
 (defun alloc-immobile-symbol ()
@@ -395,50 +404,12 @@
     (%set-symbol-package symbol nil)
     symbol))
 
-(defun alloc-immobile-fdefn ()
-  (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag fdefn-size
-                      (logior (ash (1- fdefn-size) n-widetag-bits) fdefn-widetag))))
-
-#!+immobile-code
-(progn
-(defun alloc-immobile-gf ()
-  (values (%primitive alloc-immobile-fixedobj fun-pointer-lowtag 6 ; kludge
-                      (logior (ash 5 n-widetag-bits) funcallable-instance-widetag))))
-(defun make-immobile-gf (layout slot-vector)
-  (let ((gf (truly-the funcallable-instance (alloc-immobile-gf))))
-    ;; Set layout prior to writing raw slots
-    (setf (%funcallable-instance-layout gf) layout)
-    ;; just being pedantic - liveness is preserved by the stack reference,
-    ;; and address is fixed, by definition.
-    (with-pinned-objects (gf)
-      (let ((addr (logandc2 (get-lisp-obj-address gf) lowtag-mask)))
-        (setf (sap-ref-word (int-sap addr)
-                            (ash funcallable-instance-trampoline-slot word-shift))
-              (truly-the word (+ addr (ash 4 word-shift))))))
-    (%set-funcallable-instance-info gf 0 slot-vector)
-    (!set-fin-trampoline gf)
-    gf))
-
-(defun alloc-immobile-trampoline ()
-  (let ((obj (%primitive alloc-immobile-fixedobj other-pointer-lowtag 6
-                         ;; total word count
-                         (logior (ash 6 code-header-size-shift) code-header-widetag))))
-    (with-pinned-objects (obj)
-      (setf (sap-ref-word (int-sap (get-lisp-obj-address obj))
-                          (- (ash code-boxed-size-slot word-shift)
-                             other-pointer-lowtag))
-            ;; The 'fixups' slot permanently contains 0. Therefore we can say that
-            ;; there are only 3 boxed header words instead of the usual 4 words.
-            (* 3 n-word-bytes))) ; boxed size in bytes, untagged
-    obj))
-
-) ; end PROGN
 ) ; end PROGN
 
 (declaim (inline immobile-space-addr-p))
 (defun immobile-space-addr-p (addr)
   (declare (type word addr) (ignorable addr))
-  #!+immobile-space
+  #+immobile-space
   (or (let ((start fixedobj-space-start))
         (<= start addr (truly-the word (+ start (1- fixedobj-space-size)))))
       (let ((start varyobj-space-start))
@@ -463,60 +434,67 @@
 (defun allocate-code-object (space boxed unboxed)
   (declare (ignorable space))
   (let* ((total-words
-          (the (unsigned-byte 22) ; Enforce limit on total words as well
-               (align-up (+ boxed (ceiling unboxed n-word-bytes)) 2)))
+           (the (unsigned-byte 22) ; Enforce limit on total words as well
+                (align-up (+ boxed (ceiling unboxed n-word-bytes)) 2)))
          (code
-          #!+gencgc
-          (or #!+immobile-code
-              (when (member space '(:immobile :auto))
-                ;; We don't need to inhibit GC here - ALLOCATE-IMMOBILE-OBJ does it.
-                ;; Indicate that there are initially 2 boxed words, otherwise
-                ;; immobile space GC thinks this object is freeable.
-                (allocate-immobile-obj (ash total-words word-shift)
-                                       (logior (ash total-words code-header-size-shift)
-                                               code-header-widetag)
-                                       (ash 2 n-fixnum-tag-bits)
-                                       other-pointer-lowtag
-                                       (eq space :immobile)))
+           #+gencgc
+           (or #+immobile-code
+               (when (member space '(:immobile :auto))
+                 ;; We don't need to inhibit GC here - ALLOCATE-IMMOBILE-OBJ does it.
+                 ;; Indicate that there are initially 2 boxed words, otherwise
+                 ;; immobile space GC thinks this object is freeable.
+                 (allocate-immobile-obj (ash total-words word-shift)
+                                        (logior (ash total-words code-header-size-shift)
+                                                code-header-widetag)
+                                        (ash 2 n-fixnum-tag-bits)
+                                        other-pointer-lowtag
+                                        (eq space :immobile)))
               ;;; x86-64 has a vop which is nothing more than wrapping
-              ;; pseudo-atomic around a call to alloc_code_object() in the C runtime.
-              ;; The vop is defined in such a way that it can't be inserted into
-              ;; this fuction, but instead needs an out-of-line call to a helper function
-              ;; (because it clobbers all registers and doesn't indicate that)
-              #!+(and x86-64 (not win32))
-              (alloc-dynamic-space-code total-words)
-              #!-(and x86-64 (not win32))
-              (without-gcing
-               (%make-lisp-obj
-                (alien-funcall (extern-alien "alloc_code_object"
-                                             (function unsigned (unsigned 32)))
-                               total-words))))
-          #!+cheneygc
-          (%primitive var-alloc total-words 'alloc-code
-                      ;; subtract 1 because var-alloc always adds 1 word
-                      ;; for the header, which is not right for code objects.
-                      -1 code-header-widetag other-pointer-lowtag)))
-    ;; The 1st slot beyond the header stores the boxed header size in bytes
-    ;; as an untagged number, which has the same representation as a tagged
-    ;; value denoting a word count if WORD-SHIFT = N-FIXNUM-TAG-BITS.
-    ;; This slot is allowed to be 0 prior to writing any pointer descriptors
-    ;; into the object.
-    ;; Use ATOMIC-INCF on the serialno to get automatic wraparound,
-    ;; and not because atomicity makes things deterministic, which it doesn't
-    ;; if there are several threads allocating code.
-    ;; TODO: this unnecessarily calls CODE-HEADER-SET if code cards use soft
-    ;; marking. Maybe pin the object and use (SETF SAP-REF-WORD) instead.
-    (setf (code-header-ref code code-boxed-size-slot)
+               ;; pseudo-atomic around a call to alloc_code_object() in the C runtime.
+               ;; The vop is defined in such a way that it can't be inserted into
+               ;; this fuction, but instead needs an out-of-line call to a helper function
+               ;; (because it clobbers all registers and doesn't indicate that)
+               #+(and x86-64 (not win32))
+               (alloc-dynamic-space-code total-words)
+               #-(and x86-64 (not win32))
+               (without-gcing
+                 (%make-lisp-obj
+                  (alien-funcall (extern-alien "alloc_code_object"
+                                               (function unsigned (unsigned 32)))
+                                 total-words))))
+           #+cheneygc
+           (%primitive var-alloc total-words 'alloc-code
+                       ;; subtract 1 because var-alloc always adds 1 word
+                       ;; for the header, which is not right for code objects.
+                       -1 code-header-widetag other-pointer-lowtag nil)))
+
+    (with-pinned-objects (code)
+      (let ((sap (sap+ (int-sap (get-lisp-obj-address code))
+                       (- other-pointer-lowtag))))
+        ;; The immobile space allocator pre-zeroes, and also it needs a nonzero
+        ;; value in the boxed word count because otherwise it looks like
+        ;; an immobile space page filler. So don't do any more zeroing there.
+        ;; (Could dead immobile objects be converted to use FILLER-WIDETAG instead?)
+        (unless (immobile-space-obj-p code)
+          ;; Before writing the boxed word count, zeroize up to and including 1 word
+          ;; after the boxed header so that all point words can be safely read
+          ;; by GC and so the jump table count word is 0.
+          (loop for byte-index from (ash boxed word-shift) downto (ash 2 word-shift)
+                by n-word-bytes
+                do (setf (sap-ref-word sap byte-index) 0)))
+        ;; The 1st slot beyond the header stores the boxed header size in bytes
+        ;; as an untagged number, which has the same representation as a tagged
+        ;; value denoting a word count if WORD-SHIFT = N-FIXNUM-TAG-BITS.
+        ;; This boxed-size MUST be 0 prior to writing any pointers into the object
+        ;; because the boxed words will not necessarily have been pre-zeroed;
+        ;; scavenging them prior to zeroing them out would see wild pointers.
+        (setf (sap-ref-word sap (ash code-boxed-size-slot word-shift))
+              (ash boxed word-shift))))
+
+    ;; FIXME: Sort out 64-bit and cheneygc.
+    #+(and 64-bit cheneygc)
+    (setf (code-header-ref code 0)
           (%make-lisp-obj
-           (logior (ash boxed word-shift)
-                   #!+64-bit
-                   (logand (ash (atomic-incf sb-fasl::*code-serialno*) 32)
-                           most-positive-word))))
-    ;; Is this slot assignment necessary?  Both the C and Lisp backtrace logic
-    ;; check whether debug-info is an INSTANCE before operating with the value,
-    ;; and GC doesn't care what it is. Since stores to the code go through an
-    ;; assembly routine if soft marking is enabled, it's a spurious call
-    ;; to write a value that needn't add this object to a remembered set.
-    ;; FIXME: find out what breaks if this is removed.
-    (setf (%code-debug-info code) nil)
+           (logior (ash total-words 32)
+                   sb-vm:code-header-widetag)))
     code))
