@@ -7,7 +7,14 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
+
+(defconstant sb-assem:assem-scheduler-p nil)
+(defconstant sb-assem:+inst-alignment-bytes+ 4)
+
+(defconstant +backend-fasl-file-implementation+ :alpha)
+
+(defconstant +backend-page-bytes+ 8192)
 
 (eval-when  (:compile-toplevel :load-toplevel :execute)
 
@@ -26,7 +33,6 @@
 (defconstant single-float-normal-exponent-min 1)
 (defconstant single-float-normal-exponent-max 254)
 (defconstant single-float-hidden-bit (ash 1 23))
-(defconstant single-float-trapping-nan-bit (ash 1 22))
 
 (defconstant double-float-bias 1022)
 (defconstant-eqx double-float-exponent-byte (byte 11 20)   #'equalp)
@@ -34,7 +40,6 @@
 (defconstant double-float-normal-exponent-min 1)
 (defconstant double-float-normal-exponent-max #x7FE)
 (defconstant double-float-hidden-bit (ash 1 20))
-(defconstant double-float-trapping-nan-bit (ash 1 19))
 
 (defconstant single-float-digits
   (+ (byte-size single-float-significand-byte) 1))
@@ -104,25 +109,16 @@
 ;;; Where to put the different spaces.
 ;;;
 
-#!+linux
+#+linux
 (progn
   (defconstant read-only-space-start #x20000000)
   (defconstant read-only-space-end   #x24000000))
 
-#!+osf1
-(progn
-  (defconstant read-only-space-start #x10000000)
-  (defconstant read-only-space-end   #x25000000))
-
-
 (defconstant static-space-start    #x28000000)
 (defconstant static-space-end      #x2c000000)
 
-(defconstant dynamic-0-space-start   #x30000000)
-(defconstant dynamic-0-space-end     #x3fff0000)
-
-(defconstant dynamic-1-space-start   #x40000000)
-(defconstant dynamic-1-space-end     #x4fff0000)
+(defparameter dynamic-0-space-start  #x30000000)
+(defparameter dynamic-0-space-end    #x3fff0000)
 
 ;;; FIXME nothing refers to either of these in alpha or x86 cmucl
 ;;; backend, so they could probably be removed.
@@ -142,7 +138,6 @@
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
@@ -151,7 +146,8 @@
   ;; are still needed to avoid undefined variable warnings during sbcl
   ;; build.
   single-step-around-trap
-  single-step-before-trap)
+  single-step-before-trap
+  error-trap)
 
 ;;;; static symbols
 
@@ -163,14 +159,12 @@
 ;;; space directly after the static symbols.  That way, the raw-addr
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 ;;;
-(defparameter *static-symbols*
-  (append
-   *common-static-symbols*
-   *c-callable-static-symbols*
-   '()))
+(defconstant-eqx +static-symbols+
+  `#(,@+common-static-symbols+)
+  #'equalp)
 
-(defparameter *static-funs*
-  '(length
+(defconstant-eqx +static-fdefns+
+  #(length
     two-arg-+
     two-arg--
     two-arg-*
@@ -192,4 +186,5 @@
     two-arg-xor
     two-arg-eqv
     two-arg-gcd
-    two-arg-lcm))
+    two-arg-lcm)
+  #'equalp)
