@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; Type frobbing VOPs
 
@@ -77,6 +77,16 @@
   (:generator 6
     (load-type result function (- fun-pointer-lowtag))))
 
+(define-vop (fun-header-data)
+  (:translate fun-header-data)
+  (:policy :fast-safe)
+  (:args (x :scs (descriptor-reg)))
+  (:results (res :scs (unsigned-reg)))
+  (:result-types positive-fixnum)
+  (:generator 6
+    (loadw res x 0 fun-pointer-lowtag)
+    (inst mov res (lsr res n-widetag-bits))))
+
 (define-vop (get-header-data)
   (:translate get-header-data)
   (:policy :fast-safe)
@@ -87,22 +97,6 @@
     (loadw res x 0 other-pointer-lowtag)
     (inst mov res (lsr res n-widetag-bits))))
 
-(define-vop (get-closure-length)
-  (:translate get-closure-length)
-  (:policy :fast-safe)
-  (:args (x :scs (descriptor-reg)))
-  (:results (res :scs (unsigned-reg)))
-  (:result-types positive-fixnum)
-  (:generator 6
-    (loadw res x 0 fun-pointer-lowtag)
-    (let* ((n-size-bits (integer-length short-header-max-words))
-           (lshift (- n-word-bits (+ n-size-bits n-widetag-bits))))
-      ;; (ldb (byte n-size-bits n-widetag-bits) ...)
-      ;; is best done as "shift left, shift right" discarding
-      ;; bits out both ends.
-      (inst mov res (lsl res lshift))
-      (inst mov res (lsr res (+ lshift n-widetag-bits))))))
-
 (define-vop (set-header-data)
   (:translate set-header-data)
   (:policy :fast-safe)
@@ -171,13 +165,34 @@
   (:results (sap :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 10
-    (loadw ndescr code 0 other-pointer-lowtag)
-    ;; CODE-HEADER-WIDETAG is #x38, which has the top two bits clear,
-    ;; so we don't to clear the low bits here.  If we do, use BIC.
-    (inst mov ndescr (lsr ndescr (- n-widetag-bits word-shift)))
+    (loadw ndescr code code-boxed-size-slot other-pointer-lowtag)
     (inst sub ndescr ndescr other-pointer-lowtag)
     (inst add sap code ndescr)))
 
+(eval-when (:compile-toplevel)
+  (aver (not (logtest code-header-widetag #b11000000))))
+
+(define-vop (code-trailer-ref)
+  (:translate code-trailer-ref)
+  (:policy :fast-safe)
+  (:args (code :scs (descriptor-reg) :to (:result 0))
+         (offset :scs (signed-reg) :to (:result 0)))
+  (:arg-types * fixnum)
+  (:results (res :scs (unsigned-reg) :from (:argument 0)))
+  (:result-types unsigned-num)
+  (:generator 10
+    (loadw res code 0 other-pointer-lowtag)
+    (inst mov res (lsl res 2)) ; shift out the GC bits
+    ;; Then shift right to clear the widetag, plus 2 more to the right since we just
+    ;; left-shifted to zeroize bits. Then shift left 2 to convert words to bytes.
+    ;; The '>>2' and '<<2' cancel out because we don't need to clear all 8 bits
+    ;; of the widetag, as CODE-HEADER-WIDETAG already has bits 6 and 7 clear.
+    ;; Other places assume the same, though without much commentary.
+    ;; It's brittle magic, save for the AVER above which ensures that it works.
+    (inst add res offset (lsr res n-widetag-bits))
+    (inst sub res res other-pointer-lowtag)
+    (inst ldr res (@ code res))))
+
 (define-vop (compute-fun)
   (:args (code :scs (descriptor-reg))
          (offset :scs (signed-reg unsigned-reg)))
@@ -185,14 +200,11 @@
   (:results (func :scs (descriptor-reg)))
   (:temporary (:scs (non-descriptor-reg)) ndescr)
   (:generator 10
-    (loadw ndescr code 0 other-pointer-lowtag)
-    ;; CODE-HEADER-WIDETAG is #x38, which has the top two bits clear,
-    ;; so we don't to clear the low bits here.  If we do, use BIC.
-    (inst add ndescr offset (lsr ndescr (- n-widetag-bits word-shift)))
+    (loadw ndescr code code-boxed-size-slot other-pointer-lowtag)
+    (inst add ndescr offset ndescr)
     (inst sub ndescr ndescr (- other-pointer-lowtag fun-pointer-lowtag))
     (inst add func code ndescr)))
 ;;;
-#!+symbol-info-vops
 (define-vop (symbol-info-vector)
   (:policy :fast-safe)
   (:translate symbol-info-vector)
@@ -206,7 +218,6 @@
     (inst cmp temp list-pointer-lowtag)
     (loadw res res cons-cdr-slot list-pointer-lowtag :eq)))
 
-#!+symbol-info-vops
 (define-vop (symbol-plist)
   (:policy :fast-safe)
   (:translate symbol-plist)
@@ -222,10 +233,10 @@
 
 ;;;; other miscellaneous VOPs
 
-(defknown sb!unix::receive-pending-interrupt () (values))
-(define-vop (sb!unix::receive-pending-interrupt)
+(defknown sb-unix::receive-pending-interrupt () (values))
+(define-vop (sb-unix::receive-pending-interrupt)
   (:policy :fast-safe)
-  (:translate sb!unix::receive-pending-interrupt)
+  (:translate sb-unix::receive-pending-interrupt)
   (:generator 1
     (inst debug-trap)
     (inst byte pending-interrupt-trap)
