@@ -9,23 +9,18 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!HPPA-ASM")
+(in-package "SB-HPPA-ASM")
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(reg-tn-encoding) 'sb!vm)
+  (import '(reg-tn-encoding) "SB-VM")
   ;; Imports from SB-VM into this package
-  (import '(sb!vm::zero sb!vm::registers sb!vm::float-registers
-            sb!vm::single-reg sb!vm::double-reg
-            sb!vm::complex-single-reg sb!vm::complex-double-reg
-            sb!vm::fp-single-zero sb!vm::fp-double-zero
-            sb!vm::zero-tn
-            sb!vm::null-offset sb!vm::code-offset sb!vm::zero-offset)))
-
-; normally assem-scheduler-p is t, and nil if debugging the assembler
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (setf *assem-scheduler-p* nil))
-(setf *assem-max-locations* 68) ; see number-location
+  (import '(sb-vm::zero sb-vm::registers sb-vm::float-registers
+            sb-vm::single-reg sb-vm::double-reg
+            sb-vm::complex-single-reg sb-vm::complex-double-reg
+            sb-vm::fp-single-zero sb-vm::fp-double-zero
+            sb-vm::zero-tn
+            sb-vm::null-offset sb-vm::code-offset sb-vm::zero-offset)))
 
 
 ;;;; Utility functions.
@@ -137,8 +132,6 @@
 
 ;;;; Initial disassembler setup.
 
-(setf *disassem-inst-alignment-bytes* 4)
-
 (defvar *disassem-use-lisp-reg-names* t)
 
 ; In each define-instruction the form (:dependencies ...)
@@ -147,7 +140,7 @@
 ;  src, dst and temp is passed each in loc, and can be a register
 ;  immediate or anything else.
 ; this routine will return an location-number
-; this number must be less than *assem-max-locations*
+; this number must be less than +assem-max-locations+
 (defun location-number (loc)
   (etypecase loc
     (null)
@@ -171,7 +164,7 @@
        (lambda (name)
          (cond ((null name) nil)
                (t (make-symbol (concatenate 'string "$" name)))))
-       sb!vm::*register-names*))
+       sb-vm::*register-names*))
 
 (define-arg-type reg
   :printer (lambda (value stream dstate)
@@ -430,27 +423,6 @@
   (q2   :field (byte 8 5) :value 0)
   (im5  :field (byte 5 0) :reader break-im5))
 
-(defun break-control (chunk inst stream dstate)
-  (declare (ignore inst))
-  (flet ((nt (x) (if stream (note x dstate))))
-    (case (break-im5 chunk dstate)
-      (#.error-trap
-       (nt "Error trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.cerror-trap
-       (nt "Cerror trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.breakpoint-trap
-       (nt "Breakpoint trap"))
-      (#.pending-interrupt-trap
-       (nt "Pending interrupt trap"))
-      (#.halt-trap
-       (nt "Halt trap"))
-      (#.fun-end-breakpoint-trap
-       (nt "Function end breakpoint trap"))
-      (#.single-step-around-trap
-       (nt "Single step around trap")))))
-
 (define-instruction-format (system-inst 32)
   (op1 :field (byte 6 26) :value 0)
   (r1  :field (byte 5 21) :type 'reg)
@@ -510,7 +482,7 @@
   (declare (type (or fixup (signed-byte 32) (unsigned-byte 32)) value))
   (cond ((fixup-p value)
          (note-fixup segment :hi value)
-         (aver (or (null (fixup-offset value)) (zerop (fixup-offset value))))
+         (aver (zerop (fixup-offset value)))
          0)
         (t
          (let ((hi (ldb (byte 21 11) value)))
@@ -542,7 +514,7 @@
 (defun encode-disp/fixup (segment disp imm-bits)
   (cond
     ((fixup-p disp)
-      (aver (or (null (fixup-offset disp)) (zerop (fixup-offset disp))))
+      (aver (zerop (fixup-offset disp)))
       (if imm-bits
         (note-fixup segment :load11u disp)
         (note-fixup segment :load disp))
@@ -556,6 +528,7 @@
 ; or load an 11bit-unsigned value. The latter is used for
 ; example in an LDIL/LDO pair. The key :unsigned specifies this.
 (macrolet ((define-load-inst (name opcode &optional imm-bits)
+             (declare (ignore imm-bits)) ; what?
              `(define-instruction ,name (segment disp base reg &key unsigned)
                 (:declare (type tn reg base)
                           (type (member t nil) unsigned)
@@ -620,7 +593,7 @@
   (declare (type (or fixup (signed-byte 5)) disp))
   (cond ((fixup-p disp)
          (note-fixup segment :load-short disp)
-         (aver (or (null (fixup-offset disp)) (zerop (fixup-offset disp))))
+         (aver (zerop (fixup-offset disp)))
          0)
         (t
          (dpb (ldb (byte 4 0) disp)
@@ -744,7 +717,7 @@
   (declare (type (or fixup (signed-byte 17)) disp))
   (cond ((fixup-p disp)
          (note-fixup segment :branch disp)
-         (aver (or (null (fixup-offset disp)) (zerop (fixup-offset disp))))
+         (aver (zerop (fixup-offset disp)))
          (values 0 0 0))
         (t
          (values (ldb (byte 5 11) disp)
@@ -1011,6 +984,7 @@
   (byte 1 12) (byte 1 11) (byte 11 0))
 
 (macrolet ((define-imm-inst (name cond-kind opcode subcode &optional pinned)
+             (declare (ignorable pinned))
              `(define-instruction ,name (segment imm src dst &optional cond)
                 (:declare (type tn dst src)
                   (type (signed-byte 11) imm))
@@ -1497,7 +1471,8 @@
   (:vop-var vop)
   (:emitter
    (emit-chooser segment 8 2
-     (lambda (segment posn delta)
+     (lambda (segment chooser posn delta)
+       (declare (ignore chooser))
        (let ((disp (label-relative-displacement target posn delta)))
          (when (<= 0 disp (1- (ash 1 11)))
            (assemble (segment vop)
@@ -1526,7 +1501,8 @@
   (:vop-var vop)
   (:emitter
    (emit-chooser segment 8 2
-     (lambda (segment posn delta-if-after)
+     (lambda (segment chooser posn delta-if-after)
+       (declare (ignore chooser))
        (let ((disp (label-relative-displacement target posn delta-if-after)))
          (when (and (<= 0 disp (1- (ash 1 11)))
                     (typep imm '(signed-byte 5)))
@@ -1578,7 +1554,8 @@
    ;; We emit either 12 or 4 bytes, so we maintain 3 byte alignments.
    segment 12 3
    ;; This is the best-case that emits one instruction ( 4 bytes )
-   (lambda (segment posn delta-if-after)
+   (lambda (segment chooser posn delta-if-after)
+     (declare (ignore chooser))
      (let ((delta (funcall calc label posn delta-if-after)))
        ;; WHEN, Why not AVER ?
        (when (typep delta '(signed-byte 11))
