@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 (defconstant arg-count-sc (make-sc+offset immediate-arg-scn nargs-offset))
 (defconstant closure-sc (make-sc+offset descriptor-reg-sc-number lexenv-offset))
 
@@ -115,7 +115,7 @@
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
-    (inst .skip (* (1- simple-fun-code-offset) n-word-bytes))
+    (inst .skip (* (1- simple-fun-insts-offset) n-word-bytes))
     (let ((entry-point (gen-label)))
       (emit-label entry-point)
       ;; FIXME alpha port has a ### note here saying we should "save it
@@ -234,7 +234,7 @@ default-value-8
            (type unsigned-byte nvals) (type tn move-temp temp))
   (if (<= nvals 1)
       (progn
-        (sb!assem:without-scheduling ()
+        (sb-assem:without-scheduling ()
           (note-this-location vop :single-value-return)
           (move csp-tn ocfp-tn)
           (inst nop))
@@ -243,7 +243,7 @@ default-value-8
             (defaulting-done (gen-label))
             (default-stack-vals (gen-label)))
         ;; Branch off to the MV case.
-        (sb!assem:without-scheduling ()
+        (sb-assem:without-scheduling ()
           (note-this-location vop :unknown-return)
           (if (> nvals register-arg-count)
               (inst addic. temp nargs-tn (- (fixnumize register-arg-count)))
@@ -318,7 +318,7 @@ default-value-8
   (declare (type tn args nargs start count temp))
   (let ((variable-values (gen-label))
         (done (gen-label)))
-    (sb!assem:without-scheduling ()
+    (sb-assem:without-scheduling ()
       (inst b variable-values)
       (inst nop))
 
@@ -732,17 +732,17 @@ default-value-8
                 (insert-step-instrumenting (callable-tn)
                   ;; Conditionally insert a conditional trap:
                   (when step-instrumenting
-                    ;; Get the symbol-value of SB!IMPL::*STEPPING*
-                    #!-sb-thread
-                    (load-symbol-value stepping sb!impl::*stepping*)
-                    #!+sb-thread
+                    ;; Get the symbol-value of SB-IMPL::*STEPPING*
+                    #-sb-thread
+                    (load-symbol-value stepping sb-impl::*stepping*)
+                    #+sb-thread
                     (loadw stepping thread-base-tn thread-stepping-slot)
                     (inst cmpwi stepping 0)
                     ;; If it's not null, trap.
                     (inst beq step-done-label)
                     ;; CONTEXT-PC will be pointing here when the
                     ;; interrupt is handled, not after the UNIMP.
-                    (note-this-location vop :step-before-vop)
+                    (note-this-location vop :internal-error)
                     ;; Construct a trap code with the low bits from
                     ;; SINGLE-STEP-AROUND-TRAP and the high bits from
                     ;; the register number of CALLABLE-TN.
@@ -772,7 +772,7 @@ default-value-8
                   ;; it is essential (due to it being an interior
                   ;; pointer) that the function itself be in a
                   ;; register before the raw-addr is loaded.
-                  (sb!assem:without-scheduling ()
+                  (sb-assem:without-scheduling ()
                     (loadw function name-pass fdefn-fun-slot
                         other-pointer-lowtag)
                     (loadw entry-point name-pass fdefn-raw-addr-slot
@@ -796,7 +796,7 @@ default-value-8
                   ;; is calculated.
                   (insert-step-instrumenting function)
                   (inst addi entry-point function
-                        (- (ash simple-fun-code-offset word-shift)
+                        (- (ash simple-fun-insts-offset word-shift)
                            fun-pointer-lowtag))))
                (:direct
                 `((inst lwz entry-point null-tn (static-fun-offset fun)))))
@@ -867,7 +867,7 @@ default-value-8
         (inst addi nsp-tn cur-nfp
               (- (bytes-needed-for-non-descriptor-stack-frame)
                  number-stack-displacement))))
-    (inst lr temp (make-fixup 'tail-call-variable :assembly-routine))
+    (load-asm-rtn-addr temp 'tail-call-variable)
     (inst mtlr temp)
     (inst blr)))
 
@@ -995,7 +995,7 @@ default-value-8
       (move old-fp old-fp-arg)
       (move vals vals-arg)
       (move nvals nvals-arg)
-      (inst lr temp (make-fixup 'return-multiple :assembly-routine))
+      (load-asm-rtn-addr temp 'return-multiple)
       (inst mtlr temp)
       (inst blr))))
 
@@ -1119,6 +1119,20 @@ default-value-8
   (:variant 0 0)
   (:translate %more-arg))
 
+(define-vop (more-arg-or-nil)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg) :to (:result 1))
+         (count :scs (any-reg)))
+  (:info index)
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 3
+    (inst cmpwi count (fixnumize index))
+    (move value null-tn)
+    (inst ble done)
+    (inst lwz value object (ash index word-shift))
+    done))
+
 ;;; Turn more arg (context, count) into a list.
 (define-vop (listify-rest-args)
   (:args (context-arg :target context :scs (descriptor-reg))
@@ -1198,7 +1212,7 @@ default-value-8
 ;;; below the current stack top.
 (define-vop (more-arg-context)
   (:policy :fast-safe)
-  (:translate sb!c::%more-arg-context)
+  (:translate sb-c::%more-arg-context)
   (:args (supplied :scs (any-reg)))
   (:arg-types tagged-num (:constant fixnum))
   (:info fixed)
@@ -1232,17 +1246,17 @@ default-value-8
   (:policy :fast-safe)
   (:vop-var vop)
   (:generator 3
-    ;; Get the symbol-value of SB!IMPL::*STEPPING*
-    #!-sb-thread
-    (load-symbol-value stepping sb!impl::*stepping*)
-    #!+sb-thread
+    ;; Get the symbol-value of SB-IMPL::*STEPPING*
+    #-sb-thread
+    (load-symbol-value stepping sb-impl::*stepping*)
+    #+sb-thread
     (loadw stepping thread-base-tn thread-stepping-slot)
     (inst cmpwi stepping 0)
     ;; If it's not zero, trap.
     (inst beq DONE)
     ;; CONTEXT-PC will be pointing here when the interrupt is handled,
     ;; not after the UNIMP.
-    (note-this-location vop :step-before-vop)
+    (note-this-location vop :internal-error)
     ;; CALLEE-REGISTER-OFFSET isn't needed for before-traps, so we
     ;; can just use a bare SINGLE-STEP-BEFORE-TRAP as the code.
     (inst unimp single-step-before-trap)
