@@ -13,7 +13,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; error routines
 ;;;;
@@ -80,7 +80,7 @@
 (defun listify-restrictions (restr)
   (declare (type sc-vector restr))
   (collect ((res))
-    (dotimes (i sb!vm:sc-number-limit)
+    (dotimes (i sb-vm:sc-number-limit)
       (when (eq (svref restr i) t)
         (res (svref *backend-sc-numbers* i))))
     (res)))
@@ -132,7 +132,7 @@
       (collect ((load-lose)
                 (no-move-scs)
                 (move-lose))
-        (dotimes (i sb!vm:sc-number-limit)
+        (dotimes (i sb-vm:sc-number-limit)
           (let ((i-sc (svref *backend-sc-numbers* i)))
             (when (eq (svref load-scs i) t)
               (cond ((not (sc-allowed-by-primitive-type i-sc ptype))
@@ -185,9 +185,9 @@
 ;;;; We do some checking of the consistency of the VM definition at
 ;;;; load time.
 
-;;; FIXME: should probably be conditional on #!+SB-SHOW
+;;; FIXME: should probably be conditional on #+SB-SHOW
 (defun check-move-fun-consistency ()
-  (dotimes (i sb!vm:sc-number-limit)
+  (dotimes (i sb-vm:sc-number-limit)
     (let ((sc (svref *backend-sc-numbers* i)))
       (when sc
         (let ((moves (sc-move-funs sc)))
@@ -214,8 +214,8 @@
 ;;; ignore TYPE-CHECK-ERROR because we don't want the possibility of
 ;;; error to bias the result. Notes are suppressed for T-C-E as well,
 ;;; since we don't need to worry about the efficiency of that case.
-(defparameter *ignore-cost-vops* '(set type-check-error))
-(defparameter *suppress-note-vops* '(type-check-error))
+(define-load-time-global *ignore-cost-vops* '(set type-check-error))
+(define-load-time-global *suppress-note-vops* '(type-check-error))
 
 ;;; We special-case the move VOP, since using this costs for the
 ;;; normal MOVE would spuriously encourage descriptor representations.
@@ -292,7 +292,7 @@
                             t)
 
 
-  (let ((min most-positive-fixnum)
+  (let ((min sb-xc:most-positive-fixnum)
         (min-scn nil)
         (unique nil))
     (dolist (scn scs)
@@ -303,7 +303,7 @@
                (setq min cost)
                (setq min-scn scn)
                (setq unique t)))))
-    (values (svref *backend-sc-numbers* min-scn) unique)))
+    (values min-scn unique)))
 
 ;;; Prepare for the possibility of a TN being allocated on the number
 ;;; stack by setting NUMBER-STACK-P in all functions that TN is
@@ -462,25 +462,28 @@
                        (change-tn-ref-tn op temp)
                        (cond
                          ((not write-p)
-                          (emit-move (or (maybe-move-from-fixnum+-1 op-tn temp)
-                                         res)
-                                     op-tn temp))
+                          (or
+                           (coerce-from-constant op temp)
+                           (emit-move (or (maybe-move-from-fixnum+-1 op-tn temp
+                                                                     op)
+                                          res)
+                                      op-tn temp)))
                          ((and (null (tn-reads op-tn))
                                (eq (tn-kind op-tn) :normal)))
                          (t
-                          (emit-move (or (maybe-move-from-fixnum+-1 temp op-tn op-tn)
+                          (emit-move (or (maybe-move-from-fixnum+-1 temp op-tn op)
                                          res)
                                      temp op-tn))))
                      t)))))
       ;; Search the non-stack load SCs first.
-      (dotimes (scn sb!vm:sc-number-limit)
+      (dotimes (scn sb-vm:sc-number-limit)
         (let ((sc (svref *backend-sc-numbers* scn)))
           (when (and (eq (svref scs scn) t)
                      (not (eq (sb-kind (sc-sb sc)) :unbounded))
                      (check-sc scn sc))
             (return-from emit-coerce-vop))))
       ;; Search the stack SCs if the above failed.
-      (dotimes (scn sb!vm:sc-number-limit (bad-coerce-error op))
+      (dotimes (scn sb-vm:sc-number-limit (bad-coerce-error op))
         (let ((sc (svref *backend-sc-numbers* scn)))
           (when (and (eq (svref scs scn) t)
                      (eq (sb-kind (sc-sb sc)) :unbounded)
@@ -580,36 +583,109 @@
                                 after)))))
   (values))
 
-(defun tn-type (tn)
-  (do ((type *empty-type*)
-       (ref (tn-writes tn) (tn-ref-next ref)))
-      ((null ref) type)
-    (let* ((tn (tn-ref-tn ref))
-           (node (vop-node (tn-ref-vop (tn-writes tn))))
-           (lvar (and (valued-node-p node)
-                      (node-lvar node)))
-           (locs (and lvar
-                      (ir2-lvar-locs (lvar-info lvar)))))
-      (if (and (null (cdr locs))
-               (eq (car locs) tn))
-          (setf type (type-union type
-                                 (lvar-type lvar)))
-          (return *empty-type*)))))
-
-(defun maybe-move-from-fixnum+-1 (x y &optional (original-x x))
-  (when (and (sc-is y sb!vm::descriptor-reg)
-             (sc-is x sb!vm::signed-reg sb!vm::unsigned-reg))
-    (let ((type (tn-type original-x)))
-      (cond ((eq type *empty-type*)
+(defun maybe-move-from-fixnum+-1 (x y &optional x-tn-ref)
+  (when (and (sc-is y sb-vm::descriptor-reg)
+             (sc-is x sb-vm::signed-reg sb-vm::unsigned-reg))
+    (let ((type (tn-ref-type x-tn-ref)))
+      (cond ((not type)
              nil)
             ((csubtypep type
-                        (specifier-type `(integer ,sb!xc:most-negative-fixnum
-                                                  ,(1+ sb!xc:most-positive-fixnum))))
-             (template-or-lose 'sb!vm::move-from-fixnum+1))
+                        (specifier-type `(integer ,sb-xc:most-negative-fixnum
+                                                  ,(1+ sb-xc:most-positive-fixnum))))
+             (template-or-lose 'sb-vm::move-from-fixnum+1))
             ((csubtypep type
-                        (specifier-type `(integer ,(1- sb!xc:most-negative-fixnum)
-                                                  ,sb!xc:most-positive-fixnum)))
-             (template-or-lose 'sb!vm::move-from-fixnum-1))))))
+                        (specifier-type `(integer ,(1- sb-xc:most-negative-fixnum)
+                                                  ,sb-xc:most-positive-fixnum)))
+             (template-or-lose 'sb-vm::move-from-fixnum-1))))))
+
+(defun coerce-from-constant (x-tn-ref y)
+  (when (and (sc-is y sb-vm::descriptor-reg sb-vm::control-stack)
+             (tn-ref-type x-tn-ref))
+    (multiple-value-bind (constantp value) (type-singleton-p (tn-ref-type x-tn-ref))
+      (when constantp
+        (change-tn-ref-tn x-tn-ref
+                          (make-constant-tn (find-constant value) t))
+        t))))
+
+(defun split-ir2-block (vop)
+  (cond ((vop-next vop)
+         (with-ir1-environment-from-node (vop-node vop)
+           (let* ((2block (vop-block vop))
+                  (succ (ir2-block-block 2block))
+                  (start (make-ctran))
+                  (block (ctran-starts-block start))
+                  (no-op-node (make-no-op))
+                  (new-2block (make-ir2-block block))
+                  (vop-next (vop-next vop)))
+             (link-node-to-previous-ctran no-op-node start)
+             (setf (block-info block) new-2block)
+             (add-to-emit-order new-2block (ir2-block-prev 2block))
+             (loop for pred-block in (block-pred succ)
+                   do
+                   (change-block-successor pred-block succ block))
+             (setf (block-last block) no-op-node)
+             (setf (ir2-block-start-vop new-2block) vop
+                   (ir2-block-last-vop new-2block) vop
+                   (ir2-block-start-vop 2block) vop-next)
+
+             (shiftf (ir2-block-%label new-2block)
+                     (ir2-block-%label 2block)
+                     (gen-label))
+             (shiftf (ir2-block-%trampoline-label new-2block)
+                     (ir2-block-%trampoline-label 2block)
+                     nil)
+             (setf (vop-block vop) new-2block
+                   (vop-next vop) nil
+                   (vop-prev vop-next) nil)
+             (link-blocks block succ)
+             2block)))
+        (t
+         (let* ((2block (vop-block vop))
+                (next (ir2-block-next 2block)))
+           (unless (ir2-block-%label next)
+             (setf (ir2-block-%label next) (gen-label)))
+           next))))
+
+;;; If a MOVE about to be coerced is going to another MOVE, the
+;;; result of which is compatible with the original TN, jump directly
+;;; after that move without performing any coercions.
+(defun jump-over-move-coercion (vop x y block)
+  (let* ((reads (tn-reads y))
+         (dest-vop (and reads
+                        (not (tn-ref-next reads))
+                        (tn-ref-vop reads)))
+         branch)
+    (when (and dest-vop
+               (vop-info-move-vop-p (vop-info dest-vop))
+               (eq (ir2-block-start-vop (vop-block dest-vop)) dest-vop)
+               (let ((last (ir2-block-last-vop block))
+                     (dest-block (vop-block dest-vop)))
+                 (or (and (eq last vop)
+                          (eq (ir2-block-next block) dest-block))
+                     (and last
+                          (eq (vop-next vop) last)
+                          (eq (vop-name last) 'branch)
+                          (eq (car (vop-codegen-info last)) (ir2-block-%label dest-block))
+                          (setf branch last)))))
+      (let ((dest (tn-ref-tn (vop-results dest-vop))))
+        (when (and (eq (tn-sc x) (tn-sc dest))
+                   (eq (find-move-vop x nil (tn-sc dest) (tn-primitive-type dest) #'sc-move-vops)
+                       (vop-info vop)))
+          (let ((new-block (split-ir2-block dest-vop))
+                (1block (ir2-block-block block)))
+            (if branch
+                (setf (vop-codegen-info branch) (list (ir2-block-%label new-block)))
+                (emit-and-insert-vop (vop-node vop)
+                                     block
+                                     (template-or-lose 'branch)
+                                     nil
+                                     nil
+                                     nil
+                                     (list (ir2-block-%label new-block))))
+            (change-tn-ref-tn (vop-results vop) dest)
+            (change-block-successor 1block (car (block-succ 1block))
+                                    (ir2-block-block new-block))
+            t))))))
 
 ;;; Scan the IR2 looking for move operations that need to be replaced
 ;;; with special-case VOPs and emitting coercion VOPs for operands of
@@ -622,8 +698,7 @@
             (vop-next vop)))
       ((null vop))
     (let ((info (vop-info vop))
-          (node (vop-node vop))
-          (block (vop-block vop)))
+          (node (vop-node vop)))
       (cond
         ((eq (vop-info-name info) 'move)
          (let* ((args (vop-args vop))
@@ -636,14 +711,18 @@
                        (eq (tn-kind y) :normal))
                   (delete-vop vop))
                  ((eq res info))
+                 ((coerce-from-constant args y))
                  (res
-                  (let ((res (or (maybe-move-from-fixnum+-1 x y)
-                                 res)))
-                    (when (>= (vop-info-cost res)
-                              *efficiency-note-cost-threshold*)
-                      (maybe-emit-coerce-efficiency-note res args y))
-                    (emit-move-template node block res x y vop)
-                    (delete-vop vop)))
+                  (or
+                   (jump-over-move-coercion vop x y block)
+                   (let ((res (or (maybe-move-from-fixnum+-1 x y
+                                                             args)
+                                  res)))
+                     (when (>= (vop-info-cost res)
+                               *efficiency-note-cost-threshold*)
+                       (maybe-emit-coerce-efficiency-note res args y))
+                     (emit-move-template node (vop-block vop) res x y vop)
+                     (delete-vop vop))))
                  (t
                   (coerce-vop-operands vop)))))
         ((vop-info-move-args info)
@@ -654,7 +733,7 @@
 ;;; If TN is in a number stack SC, make all the right annotations.
 ;;; Note that this should be called after TN has been referenced,
 ;;; since it must iterate over the referencing environments.
-#!-sb-fluid (declaim (inline note-if-number-stack))
+#-sb-fluid (declaim (inline note-if-number-stack))
 (defun note-if-number-stack (tn 2comp restricted)
   (declare (type tn tn) (type ir2-component 2comp))
   (when (if restricted
@@ -676,43 +755,46 @@
 ;;; allocated. This must be done last, since references in new
 ;;; environments may be introduced by MOVE-ARG insertion.
 (defun select-representations (component)
-  (let ((costs (make-array sb!vm:sc-number-limit))
+  (let ((costs (make-array sb-vm:sc-number-limit))
         (2comp (component-info component)))
-    (labels ((assign-constant-offset (tn)
-               (when (sc-is tn constant)
-                 (setf (tn-offset tn)
-                       (or (position (tn-leaf tn)
-                                     (ir2-component-constants 2comp))
-                           (vector-push-extend (tn-leaf tn)
-                                               (ir2-component-constants 2comp))))))
+    (labels ((set-sc (tn sc)
+               (cond ((not (eq (tn-kind tn) :constant))
+                      (setf (tn-sc tn)
+                            (svref *backend-sc-numbers* sc)))
+                     ;; Translate primitive type scs into constant scs
+                     ((= sc sb-vm::descriptor-reg-sc-number)
+                      (setf (tn-sc tn)
+                            (svref *backend-sc-numbers* sb-vm:constant-sc-number)
+                            (tn-offset tn)
+                            (or (position (tn-leaf tn)
+                                          (ir2-component-constants 2comp))
+                                (vector-push-extend (tn-leaf tn)
+                                                    (ir2-component-constants 2comp)))))
+                     (t
+                      (setf (tn-sc tn)
+                            (svref *backend-sc-numbers*
+                                   (immediate-constant-sc (constant-value (tn-leaf tn))))))))
              (possible-scs (tn)
-               (if (eq (tn-kind tn) :constant)
-                   (list sb!vm:constant-sc-number
-                         (immediate-constant-sc (constant-value (tn-leaf tn))))
-                   (primitive-type-scs (tn-primitive-type tn))))
+               (primitive-type-scs (tn-primitive-type tn)))
              (pass (tn &key unique)
                (do ((tn tn (tn-next tn)))
                    ((null tn))
                  (aver (tn-primitive-type tn))
                  (unless (tn-sc tn)
-                   (let* ((scs (possible-scs tn)))
+                   (let ((scs (possible-scs tn)))
                      (cond ((rest scs)
                             (multiple-value-bind (sc uniquep)
                                 (select-tn-representation tn scs costs)
                               (when (or (not unique)
                                         uniquep)
-                                (setf (tn-sc tn) sc)
-                                (assign-constant-offset tn))))
+                                (set-sc tn sc))))
                            (t
-                            (setf (tn-sc tn)
-                                  (svref *backend-sc-numbers* (first scs)))
-                            (assign-constant-offset tn))))))))
+                            (set-sc tn (first scs)))))))))
 
       ;; First pass; only allocate SCs where there is a distinct choice.
-      (pass (ir2-component-constant-tns 2comp) :unique t)
       (pass (ir2-component-normal-tns 2comp) :unique t)
-      (pass (ir2-component-constant-tns 2comp))
-      (pass (ir2-component-normal-tns 2comp)))
+      (pass (ir2-component-normal-tns 2comp))
+      (pass (ir2-component-constant-tns 2comp)))
 
     (do ((alias (ir2-component-alias-tns 2comp)
                 (tn-next alias)))
@@ -721,6 +803,9 @@
 
     (do-ir2-blocks (block component)
       (emit-moves-and-coercions block))
+    ;; Give the optimizers a second opportunity to alter newly inserted vops
+    ;; by looking for patterns that have a shorter expression as a single vop.
+    (run-vop-optimizers component)
 
     (macrolet ((frob (slot restricted)
                  `(do ((tn (,slot 2comp) (tn-next tn)))
