@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (defun zeroize (tn)
   (inst xor :dword tn tn))
@@ -78,13 +78,13 @@
   ;; is already zeroed. Otherwise a :qword.
   (cond
     ;; If target is a register, we can just mov it there directly
-    ((gpr-p target)
+    ((gpr-tn-p target)
      ;; val can be a fixup for an immobile-space symbol, i.e. not a number,
      ;; hence not acceptable to ZEROP.
      (cond ((and (numberp val) (zerop val)) (zeroize target))
            (t (inst mov target val))))
     ;; Likewise if the value is small enough.
-    ((typep val '(or (signed-byte 32) #!+immobile-space fixup))
+    ((typep val '(or (signed-byte 32) #+immobile-space fixup))
      ;; This logic is similar to that of STOREW*.
      ;; It would be nice to pull it all together in one place.
      ;; The basic idea is that storing any byte-aligned 8-bit value
@@ -95,8 +95,11 @@
                      (typecase val
                        ((unsigned-byte 8) :byte)
                        ((unsigned-byte 16) :word)
+                       ;; fixups can be :dword size because we don't reference
+                       ;; objects that require a 64-bit address as immediate operands.
                        ;; signed-32 is no good, as it needs sign-extension.
-                       ((unsigned-byte 32) :dword)))
+                       ((or (unsigned-byte 32) fixup)
+                        :dword)))
                 :qword)))
        (inst mov operand-size target val)))
     ;; Otherwise go through the temporary register
@@ -166,7 +169,7 @@
   (:results (y :scs (signed-reg unsigned-reg)))
   (:note "constant load")
   (:generator 1
-    (cond ((sb!c::tn-leaf x)
+    (cond ((sb-c::tn-leaf x)
            (inst mov y (tn-value x)))
           (t
            (inst mov y x)
@@ -176,7 +179,7 @@
 
 
 ;;; Arg is a fixnum or bignum, figure out which and load if necessary.
-#-#.(cl:if (cl:= sb!vm:n-fixnum-tag-bits 1) '(:and) '(:or))
+#-#.(cl:if (cl:= sb-vm:n-fixnum-tag-bits 1) '(:and) '(:or))
 (define-vop (move-to-word/integer)
   (:args (x :scs (descriptor-reg) :target rax))
   (:results (y :scs (signed-reg unsigned-reg)))
@@ -188,7 +191,7 @@
                :from (:argument 0) :to (:result 0) :target y) rax)
   (:generator 4
     (move rax x)
-    (inst test al-tn fixnum-tag-mask)
+    (inst test :byte rax fixnum-tag-mask)
     (inst jmp :z FIXNUM)
     (loadw y rax bignum-digits-offset other-pointer-lowtag)
     (inst jmp DONE)
@@ -197,7 +200,7 @@
     (move y rax)
     DONE))
 
-#+#.(cl:if (cl:= sb!vm:n-fixnum-tag-bits 1) '(:and) '(:or))
+#+#.(cl:if (cl:= sb-vm:n-fixnum-tag-bits 1) '(:and) '(:or))
 (define-vop (move-to-word/integer)
   (:args (x :scs (descriptor-reg) :target y))
   (:results (y :scs (signed-reg unsigned-reg)))
@@ -252,7 +255,7 @@
                     (unless (member x '(rsp rbp) :test 'string=)
                       (symbolicate "ALLOC-" signedp "-BIGNUM-IN-" x)))
                   +qword-register-names+)
-           (ash (tn-offset ,tn) -1))))
+           (tn-offset ,tn))))
 
 ;;; Convert an untagged signed word to a lispobj -- fixnum or bignum
 ;;; as the case may be. Fixnum case inline, bignum case in an assembly
@@ -287,46 +290,58 @@
   (:args (x :scs (signed-reg unsigned-reg)))
   (:results (y :scs (any-reg descriptor-reg)))
   (:generator 4
-    #.(aver (= n-fixnum-tag-bits 1))
-    (move y x)
-    (inst shl y 1)
-    (inst cmov :o y (emit-constant (1+ sb!xc:most-positive-fixnum)))))
+    (cond ((= n-fixnum-tag-bits 1)
+           (move y x)
+           (inst shl y 1)
+           (inst cmov :o y (emit-constant (1+ sb-xc:most-positive-fixnum))))
+          (t
+           ;; not worth optimizing into SHL. The processor doesn't set OF for
+           ;; for shift count > 1 so we'd have to detect overflow differently.
+           (inst imul y x (ash 1 n-fixnum-tag-bits))
+           (inst jmp :no DONE)
+           (move y (emit-constant (1+ sb-xc:most-positive-fixnum)))))
+    DONE))
 
 (define-vop (move-from-fixnum-1 move-from-fixnum+1)
   (:generator 4
-    #.(aver (= n-fixnum-tag-bits 1))
-    (move y x)
-    (inst shl y 1)
-    (inst cmov :o y (emit-constant (1- sb!xc:most-negative-fixnum)))))
+    (cond ((= n-fixnum-tag-bits 1)
+           (move y x)
+           (inst shl y 1)
+           (inst cmov :o y (emit-constant (1- sb-xc:most-negative-fixnum))))
+          (t
+           ;; not worth optimizing into SHL. The processor doesn't set OF for
+           ;; for shift count > 1 so we'd have to detect overflow differently.
+           (inst imul y x (ash 1 n-fixnum-tag-bits))
+           (inst jmp :no DONE)
+           (move y (emit-constant (1- sb-xc:most-negative-fixnum)))))
+    DONE))
 
 ;;; Convert an untagged unsigned word to a lispobj -- fixnum or bignum
 ;;; as the case may be. Fixnum case inline, bignum case in an assembly
 ;;; routine.
 (define-vop (move-from-unsigned)
-  (:args (x :scs (signed-reg unsigned-reg) :to :result))
-  (:results (y :scs (any-reg descriptor-reg) :from :argument))
+  ;; This sure seems wonky - how would this be selected for a signed arg?
+  (:args (arg :scs (signed-reg unsigned-reg signed-stack unsigned-stack)))
+  (:results (res :scs (any-reg descriptor-reg)))
   (:note "unsigned word to integer coercion")
   (:vop-var vop)
   ;; Worst case cost to make sure people know they may be number consing.
   (:generator 20
-              (aver (not (location= x y)))
-              (inst mov y #.(ash (1- (ash 1 (1+ n-fixnum-tag-bits)))
-                                 n-positive-fixnum-bits))
-              ;; The assembly routines test the sign flag from this one, so if
-              ;; you change stuff here, make sure the sign flag doesn't get
-              ;; overwritten before the CALL!
-              (inst test x y)
-              ;; Using LEA is faster but bigger than MOV+SHL; it also doesn't
-              ;; twiddle the sign flag.  The cost of doing this speculatively
-              ;; should be noise compared to bignum consing if that is needed
-              ;; and saves one branch.
-              (if (= n-fixnum-tag-bits 1)
-                  (inst lea y (ea x x))
-                  (inst lea y (ea nil x (ash 1 n-fixnum-tag-bits))))
-              (inst jmp :z done)
-              (inst mov y x)
-              (invoke-asm-routine 'call #.(bignum-from-reg 'y "UNSIGNED") vop)
-              DONE))
+    (move res arg)
+    ;; the number of high bits that need to be zero is the number of bits of
+    ;; precision lost due to fixnum tag, plus the sign bit.
+    ;; rotate those into the least significant bits.
+    (inst rol res (1+ n-fixnum-tag-bits))
+    (inst test :byte res (1- (ash 1 (1+ n-fixnum-tag-bits))))
+    ;; this could make use of the SHRX instruction which is only available with BMI2
+    ;; support- speculatively restore the fixnum value, test previous flags and jump,
+    ;; which would eliminate the 'jmp done' that jumps over the following SHR.
+    (inst jmp :z fixnum)
+    (invoke-asm-routine 'call #.(bignum-from-reg 'res "UNSIGNED") vop)
+    (inst jmp done)
+    FIXNUM
+    (inst shr res 1)
+    DONE))
 (define-move-vop move-from-unsigned :move
   (unsigned-reg) (descriptor-reg))
 
