@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; allocator for the array header
 
@@ -96,23 +96,43 @@
   (:variant t)
   (:vop-var vop)
   (:save-p :compute-only)
-  (:generator 5
+  (:generator 6
     (let ((error (generate-error-code vop 'invalid-array-index-error
                                       array bound index))
           (bound (if (sc-is bound immediate)
-                     (fixnumize (tn-value bound))
+                     (let ((value (tn-value bound)))
+                       (cond ((and %test-fixnum
+                                   (power-of-two-limit-p (1- value)))
+                              (lognot (fixnumize (1- value))))
+                             ((sc-is index any-reg descriptor-reg)
+                              (fixnumize value))
+                             (t
+                              value)))
                      bound))
           (index (if (sc-is index immediate)
-                     (fixnumize (tn-value index))
+                     (let ((value (tn-value index)))
+                       (if (sc-is bound any-reg descriptor-reg)
+                           (fixnumize value)
+                           value))
                      index)))
-      (when (and %test-fixnum (not (integerp index)))
-        (%test-fixnum index nil error t))
-      (cond ((integerp bound)
-             (inst cmp index bound)
-             (inst jmp :nb error))
+      (cond ((typep bound '(integer * -1))
+             ;; Power of two bound, can be checked for fixnumness at
+             ;; the same time as it always occupies a consecutive bit
+             ;; range, everything else, including the tag, has to be
+             ;; zero.
+             (inst test index (if (eql bound -1)
+                                  index ;; zero?
+                                  bound))
+             (inst jmp :ne error))
             (t
-             (inst cmp bound index)
-             (inst jmp :be error))))))
+             (when (and %test-fixnum (not (integerp index)))
+               (%test-fixnum index nil error t))
+             (cond ((integerp bound)
+                    (inst cmp index bound)
+                    (inst jmp :nb error))
+                   (t
+                    (inst cmp bound index)
+                    (inst jmp :be error))))))))
 
 (define-vop (check-bound/fast check-bound)
   (:policy :fast)
@@ -126,6 +146,15 @@
   (:arg-types * * tagged-num)
   (:variant nil)
   (:variant-cost 4))
+
+(define-vop (check-bound/untagged check-bound)
+  (:args (array)
+         (bound :scs (unsigned-reg signed-reg))
+         (index :scs (unsigned-reg signed-reg)))
+  (:arg-types * (:or unsigned-num signed-num)
+                (:or unsigned-num signed-num))
+  (:variant nil)
+  (:variant-cost 5))
 
 ;;;; accessors/setters
 
@@ -149,7 +178,7 @@
     signed-reg)
   (def-full-data-vector-frobs simple-array-unsigned-byte-31 unsigned-num
     unsigned-reg)
-  #!+sb-unicode
+  #+sb-unicode
   (def-full-data-vector-frobs simple-character-string character character-reg))
 
 (define-full-compare-and-swap %compare-and-swap-svref simple-vector
@@ -343,7 +372,7 @@
 
 ;;; And the float variants.
 
-(defun make-ea-for-float-ref (object index offset element-size
+(defun float-ref-ea (object index offset element-size
                               &key (scale 1) (complex-offset 0))
   (sc-case index
     (immediate
@@ -373,7 +402,7 @@
   (:result-types single-float)
   (:generator 5
    (with-empty-tn@fp-top(value)
-     (inst fld (make-ea-for-float-ref object index offset 4)))))
+     (inst fld (float-ref-ea object index offset 4)))))
 
 (define-vop (data-vector-set-with-offset/simple-array-single-float)
   (:note "inline array store")
@@ -392,14 +421,14 @@
   (:generator 5
     (cond ((zerop (tn-offset value))
            ;; Value is in ST0.
-           (inst fst (make-ea-for-float-ref object index offset 4))
+           (inst fst (float-ref-ea object index offset 4))
            (unless (zerop (tn-offset result))
              ;; Value is in ST0 but not result.
              (inst fst result)))
           (t
            ;; Value is not in ST0.
            (inst fxch value)
-           (inst fst (make-ea-for-float-ref object index offset 4))
+           (inst fst (float-ref-ea object index offset 4))
            (cond ((zerop (tn-offset result))
                   ;; The result is in ST0.
                   (inst fst value))
@@ -424,7 +453,7 @@
   (:result-types double-float)
   (:generator 7
    (with-empty-tn@fp-top(value)
-     (inst fldd (make-ea-for-float-ref object index offset 8 :scale 2)))))
+     (inst fldd (float-ref-ea object index offset 8 :scale 2)))))
 
 (define-vop (data-vector-set-with-offset/simple-array-double-float)
   (:note "inline array store")
@@ -443,14 +472,14 @@
   (:generator 20
     (cond ((zerop (tn-offset value))
            ;; Value is in ST0.
-           (inst fstd (make-ea-for-float-ref object index offset 8 :scale 2))
+           (inst fstd (float-ref-ea object index offset 8 :scale 2))
            (unless (zerop (tn-offset result))
                    ;; Value is in ST0 but not result.
                    (inst fstd result)))
           (t
            ;; Value is not in ST0.
            (inst fxch value)
-           (inst fstd (make-ea-for-float-ref object index offset 8 :scale 2))
+           (inst fstd (float-ref-ea object index offset 8 :scale 2))
            (cond ((zerop (tn-offset result))
                   ;; The result is in ST0.
                   (inst fstd value))
@@ -477,11 +506,11 @@
   (:generator 5
     (let ((real-tn (complex-single-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fld (make-ea-for-float-ref object index offset 8 :scale 2))))
+        (inst fld (float-ref-ea object index offset 8 :scale 2))))
     (let ((imag-tn (complex-single-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
         ;; FIXME
-        (inst fld (make-ea-for-float-ref object index offset 8
+        (inst fld (float-ref-ea object index offset 8
                                          :scale 2 :complex-offset 4))))))
 
 (define-vop (data-vector-set-with-offset/simple-array-complex-single-float)
@@ -503,14 +532,14 @@
           (result-real (complex-single-reg-real-tn result)))
       (cond ((zerop (tn-offset value-real))
              ;; Value is in ST0.
-             (inst fst (make-ea-for-float-ref object index offset 8 :scale 2))
+             (inst fst (float-ref-ea object index offset 8 :scale 2))
              (unless (zerop (tn-offset result-real))
                ;; Value is in ST0 but not result.
                (inst fst result-real)))
             (t
              ;; Value is not in ST0.
              (inst fxch value-real)
-             (inst fst (make-ea-for-float-ref object index offset 8 :scale 2))
+             (inst fst (float-ref-ea object index offset 8 :scale 2))
              (cond ((zerop (tn-offset result-real))
                     ;; The result is in ST0.
                     (inst fst value-real))
@@ -522,7 +551,7 @@
     (let ((value-imag (complex-single-reg-imag-tn value))
           (result-imag (complex-single-reg-imag-tn result)))
       (inst fxch value-imag)
-      (inst fst (make-ea-for-float-ref object index offset 8
+      (inst fst (float-ref-ea object index offset 8
                                        :scale 2 :complex-offset 4))
       (unless (location= value-imag result-imag)
         (inst fst result-imag))
@@ -543,10 +572,10 @@
   (:generator 7
     (let ((real-tn (complex-double-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fldd (make-ea-for-float-ref object index offset 16 :scale 4)))
+        (inst fldd (float-ref-ea object index offset 16 :scale 4)))
     (let ((imag-tn (complex-double-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
-        (inst fldd (make-ea-for-float-ref object index offset 16
+        (inst fldd (float-ref-ea object index offset 16
                                           :scale 4 :complex-offset 8)))))))
 
 (define-vop (data-vector-set-with-offset/simple-array-complex-double-float)
@@ -568,7 +597,7 @@
           (result-real (complex-double-reg-real-tn result)))
       (cond ((zerop (tn-offset value-real))
              ;; Value is in ST0.
-             (inst fstd (make-ea-for-float-ref object index offset 16
+             (inst fstd (float-ref-ea object index offset 16
                                                :scale 4))
              (unless (zerop (tn-offset result-real))
                ;; Value is in ST0 but not result.
@@ -576,7 +605,7 @@
             (t
              ;; Value is not in ST0.
              (inst fxch value-real)
-             (inst fstd (make-ea-for-float-ref object index offset 16
+             (inst fstd (float-ref-ea object index offset 16
                                                :scale 4))
              (cond ((zerop (tn-offset result-real))
                     ;; The result is in ST0.
@@ -589,7 +618,7 @@
     (let ((value-imag (complex-double-reg-imag-tn value))
           (result-imag (complex-double-reg-imag-tn result)))
       (inst fxch value-imag)
-      (inst fstd (make-ea-for-float-ref object index offset 16
+      (inst fstd (float-ref-ea object index offset 16
                                         :scale 4 :complex-offset 8))
       (unless (location= value-imag result-imag)
         (inst fstd result-imag))
@@ -666,8 +695,8 @@
   (define-data-vector-frobs simple-array-signed-byte-8 tagged-num
     movsx nil signed-reg)
   (define-data-vector-frobs simple-base-string character
-                            #!+sb-unicode movzx #!-sb-unicode mov
-                            #!+sb-unicode nil #!-sb-unicode t character-reg))
+                            #+sb-unicode movzx #-sb-unicode mov
+                            #+sb-unicode nil #-sb-unicode t character-reg))
 
 ;;; {un,}signed-byte-16
 (macrolet ((define-data-vector-frobs (ptype element-type ref-inst &rest scs)
